library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use std.textio.all;
use ieee.numeric_std.all;



entity tb_FIRFilter_ReadFromFile is
--  Port ( );
end tb_FIRFilter_ReadFromFile;

architecture Behavioral of tb_FIRFilter_ReadFromFile is

component LMS is
    generic(
              NBTAP   : natural := 32;  
              DSIZE   : natural := 24;
              COESIZE : natural := 16
           );
      port (
              clk_100_MHz : in  std_logic;
              x_Data      : in  std_logic_vector(DSIZE - 1 downto 0);
              d_Data      : in  std_logic_vector(DSIZE - 1 downto 0);
              e_Data      : out std_logic_vector(DSIZE - 1 downto 0);
              y_Data      : out std_logic_vector(DSIZE - 1 downto 0);
              new_sample  : in  std_logic  
          ); 
end component;

component i2s_data_interface 
Port( 
      i2s_MCLK               : in  std_logic;
      filtered_data_left     : in  std_logic_vector(23 downto 0);
      filtered_data_right    : in  std_logic_vector(23 downto 0);
      unfiltered_data_left   : out std_logic_vector(23 downto 0);
      unfiltered_data_right  : out std_logic_vector(23 downto 0);
      new_sample             : out std_logic;
      i2s_SCLK               : in  std_logic;
      i2s_DA_SDIN            : out std_logic;
      i2s_AD_SDOUT           : in  std_logic;
      i2s_LRCK               : in  std_logic
    );
end component;

component clk_wiz_0 port(
  -- Clock out ports
  clk_out1          : out    std_logic;
  clk_out2          : out    std_logic;
  -- Status and control signals
  reset             : in     std_logic;
  locked            : out    std_logic;
  -- Clock in ports
  clk_in1_p         : in     std_logic;
  clk_in1_n         : in     std_logic);
end component;


signal  s_clk_100_MHz    : std_logic := '0';
signal  s_x_Data         : std_logic_vector(23 downto 0) := x"000000";
signal  s_d_Data         : std_logic_vector(23 downto 0) := x"000000";
signal  s_e_Data         : std_logic_vector(23 downto 0) := x"000000";
signal  s_y_Data         : std_logic_vector(23 downto 0) := x"000000";


signal  s_new_sample     : std_logic := '0';
--****************CDC Signals***************************
signal r1_Pulse : std_logic := '0';
signal r2_Pulse : std_logic := '0';
signal r3_Pulse : std_logic := '0';
signal s_new_sample_rising_edge  : std_logic := '0'; 

------------------------------------------------------
signal s_filtered_data_left    : std_logic_vector(23 downto 0) := (others => '0'); 
signal s_filtered_data_right   : std_logic_vector(23 downto 0) := (others => '0'); 
------------------------------------------------------
signal s_unfiltered_data_left  : std_logic_vector(23 downto 0) := (others => '0'); 
signal s_unfiltered_data_right : std_logic_vector(23 downto 0) := (others => '0'); 
------------------------------------------------------
signal s_clk_8_192_MHz_MCLK : std_logic:='0';
signal s_clk_2_048_MHz_SCLK : std_logic:='0';
signal s_clk_32_kHz_LRCK    : std_logic:='0';
signal clkdiv               : unsigned(7 downto 0):=(others => '0');
------------------------------------------------------
signal s_sysclk_p : std_logic:='0';
signal s_sysclk_n : std_logic:='0';
------------------------------------------------------
file textfile_in    : text open read_mode   is "DataX1Q23.txt";
file textfile_out   : text open write_mode  is "DataEQ23.txt"; 

file textfile_in_2     : text open read_mode   is "DataD1Q23.txt";
file textfile_out_2    : text open write_mode  is "DataYQ23.txt";

begin


process 
    begin
  wait until rising_edge(s_clk_8_192_MHz_MCLK);
  clkdiv <= clkdiv +1;
end process;
s_clk_2_048_MHz_SCLK  <= clkdiv(1);
s_clk_32_kHz_LRCK     <= clkdiv(7);


process
  variable line_in: line;
  variable line_out: line;
  variable number_in: integer;
  variable number_in_valid: boolean;
  variable number_out: integer;

    begin
      while not endfile(textfile_in) loop
        number_in_valid := TRUE;
        readline(textfile_in, line_in);
        while number_in_valid loop
          read(line_in, number_in, number_in_valid);
          exit when not number_in_valid;
          report "Wert: " & integer'image(number_in);
          s_x_Data <= std_logic_vector(to_signed(number_in, 24)); 
          number_out := to_integer(signed(s_e_Data));
          write(line_out, number_out, right, 24); 
        end loop;    
        writeline(textfile_out ,line_out); 
        wait until rising_edge(s_new_sample_rising_edge); 
        report "Next Line";
      end loop;
      report "Done";
      wait;
end process;
--***************************************************************************
process
  variable line_in_2: line;
  variable line_out_2: line;
  variable number_in_2: integer;
  variable number_in_valid_2: boolean;
  variable number_out_2: integer;

  begin
      while not endfile(textfile_in_2) loop
        number_in_valid_2 := TRUE;
        readline(textfile_in_2, line_in_2);
        while number_in_valid_2 loop
          read(line_in_2, number_in_2, number_in_valid_2);
          exit when not number_in_valid_2;
          report "Wert: " & integer'image(number_in_2);
          s_d_Data <= std_logic_vector(to_signed(number_in_2, 24)); 
          number_out_2 := to_integer(signed(s_y_Data));
          write(line_out_2, number_out_2, right, 24); 
        end loop;
        writeline(textfile_out_2 ,line_out_2); 
        wait until rising_edge(s_new_sample_rising_edge); 
        report "Next Line";
      end loop;
      report "Done";
      wait;
end process;

uut: LMS
generic map(
              NBTAP     => 32,  
              DSIZE     => 24,
              COESIZE   => 16
           )
port map (      
              clk_100_MHz => s_clk_100_MHz,
              x_Data      => s_x_Data,
              d_Data      => s_d_Data,
              e_Data      => s_e_Data,
              y_Data      => s_y_Data,
              new_sample  => s_new_sample_rising_edge  
          );
             


ck_proc : process  -- Generate 200 MHz Clock (T=5ns)
begin
    s_sysclk_p <= '1';
    s_sysclk_n <= '0';
    loop
        wait for 2.5 ns;
        s_sysclk_p <= not s_sysclk_p;
        s_sysclk_n <= not s_sysclk_n;
    end loop;
    wait;
end process ck_proc;


CDC_FAST_CLOCK_PROC : process (s_clk_100_MHz)
    begin
        if rising_edge(s_clk_100_MHz) then         
            s_new_sample_rising_edge <= '0';

            r1_Pulse <= s_new_sample;
            r2_Pulse <= r1_Pulse;
            r3_Pulse <= r2_Pulse;
            
            -- Rising Edge
            if (r3_Pulse = '0' and r2_Pulse = '1') then
                s_new_sample_rising_edge <= '1';
            end if;          
        end if;
end process;


i2s_interface : i2s_data_interface 
port map(
          i2s_MCLK              => s_clk_8_192_MHz_MCLK,
          filtered_data_left    => s_filtered_data_left,  -- std_logic_vector as Input for conversion as serial output (left channel part)
          filtered_data_right   => s_filtered_data_right,  -- std_logic_vector as Input for conversion as serial output (right channel part)
          unfiltered_data_left  => s_unfiltered_data_left, -- converted input data
          unfiltered_data_right => s_unfiltered_data_right, -- converted input data 
          new_sample            => s_new_sample,
          i2s_SCLK              => s_clk_2_048_MHz_SCLK,
          i2s_DA_SDIN           => open,
          i2s_AD_SDOUT          => '1',
          i2s_LRCK              => s_clk_32_kHz_LRCK
  );

clk_wiz : clk_wiz_0 
port map( 
          -- Clock out ports  
          clk_out1  => s_clk_100_MHz,
          clk_out2  => s_clk_8_192_MHz_MCLK, 
          -- Status and control signals                
          reset     => '0',
          locked    => open,
          -- Clock in ports
          clk_in1_p => s_sysclk_p,
          clk_in1_n => s_sysclk_n
        );


end Behavioral;



